import jphelper.constant as consts
from math import floor, log


def to_japanese(number, use_kanji=False, decimal_limit=5, separator='', minus_sign=None):
    """Convert arabic numeral to hiragana (or kanji).

    :param float number: Real number.
    :param bool use_kanji: Use kanji instead of hiragana as output.
    :param int decimal_limit: Number of decimal places to be included in output.
    :param str separator: Separator character for each unit to aid reading (default empty character).
    :param str minus_sign: Custom minus sign (default katakana version).
    :return: Japanese reading of input number.
    :rtype: str
    """
    res = ''
    decimals_str = ''
    if isinstance(number, int):
        number = float(number)
    if number < 0:
        res = consts.minus if minus_sign is None else minus_sign
        number *= -1
    if number > consts.MAX_VAL:
        raise OverflowError('Maximum value ' + consts.MAX_VAL)

    idx = 1 if use_kanji else 0
    if not float.is_integer(number) and decimal_limit > 0:
        tmp = int((number - int(number))*10**decimal_limit)
        while tmp % 10 == 0:
            tmp = tmp // 10
        tmp = [int(x) for x in str(tmp)]
        for t in tmp:
            # decimals_str += consts.unit[t][idx]
            decimals_str = _join(separator, [decimals_str, consts.unit[t][idx]])
        decimals_str = _join(separator, [consts.period[idx], decimals_str])

    number = int(number)
    power = floor(log(number, 10)) if number != 0 else 0
    prevs = False
    while power >= 1:
        dv, number = divmod(number, 10**power)
        ten = power % 4
        if ten == 0:
            ten = power
        if dv > 0:
            if dv*(10**ten) in consts.specials.keys():
                # res += consts.specials[dv*(10**ten)][idx]
                res = _join(separator, [res, consts.specials[dv*(10**ten)][idx]])
            else:
                # res += consts.unit[dv][idx] + consts.tens[10**ten][idx]
                res = _join(separator, [res, consts.unit[dv][idx], consts.tens[ten][idx]])
            prevs = True
        elif prevs:
            if res != '' and power % 4 == 0:
                # res += consts.tens[10**ten][idx]
                res = _join(separator, [res, consts.tens[ten][idx]])
                prevs = False
        power -= 1
    if 0 < number < 10 or res == '':
        # res += consts.unit[number][idx]
        res = _join(separator, [res, consts.unit[number][idx]])
    return _join(separator, [res, decimals_str])


def group_unit(number, separator=None, decimal_separator='.'):
    """Group number by ten thousandth.

    :param number: Real number to be groupped.
    :param separator: Group separator.
    :param decimal_separator: Decimal separator.
    :return: Number grouped every 4 digits except decimal parts.
    :rtype: str
    """
    res = ''
    minus = ''
    decimals = ''
    if separator is None:
        separator = consts.SEPARATOR
    if number < 0:
        minus = '-'
        number *= -1
    if isinstance(number, float):
        decimals = decimal_separator + str(number).split('.')[1]
        number = int(number)
    number = str(number)
    for i in range(len(number) - 1):
        res += number[i]
        if (len(number) - i - 1) % 4 == 0:
            res += separator
    res += number[-1]
    return minus + res + decimals


def kanji_grouping(number, use_minus_sign=True, use_hiragana=False, decimal_separator=None):
    """Add kanji/hiragana every multiple of ten thousands unit.

    :param float number: Real number.
    :param bool use_minus_sign: If True use '-' as minus sign, else use katakana version.
    :param bool use_hiragana: Use hiragana in place of kanji.
    :param str decimal_separator: Decimal separator.
    :return: Number with multipe of ten thousands unit character inserted.
    :rtype: str
    """
    idx = 0 if use_hiragana else 1
    minus, decimals = '', ''
    if number < 0:
        minus = '-' if use_minus_sign else consts.minus
        number *= -1
    if isinstance(number, float):
        decimals = consts.period[idx] + str(number).split('.')[1]
        number = int(number)
    number = str(number)
    res = ''
    start = 5 if len(number) > 4 and number[-1:-5:-1] == '0000' else 1
    for i in range(start, len(number) + 1):
        if i > 4 and (i - 1) % 4 == 0:
            res = consts.tens[i-1][idx] + res
        res = number[-i] + res
    return minus + res + decimals


def shorten(number, use_minus_sign=True, use_hiragana=False, decimal_places=1, decimal_separator=None):
    """Round number based on highest unit of 10e4.

    :param float number: Real number.
    :param bool use_minus_sign: If True use '-' as minus sign, else use katakana version.
    :param bool use_hiragana: Use hiragana in place of kanji.
    :param int decimal_places: Number of decimals included.
    :param str decimal_separator: Decimal separator.
    :return: Formatted number.
    :rtype: str
    """
    idx = 0 if use_hiragana else 1
    res, minus = '', ''
    if number < 0:
        minus = '-' if use_minus_sign else consts.minus
        number *= -1
    if decimal_separator is None:
        decimal_separator = consts.SEPARATOR
    number = str(int(number))
    if int(number) < 10000:
        res = number
    else:
        n_digit = len(number)
        taken = n_digit % 4
        unit = (n_digit // 4) * 4
        res = number[0:taken]
        if decimal_places > 0:
            decimals = number[taken:taken+decimal_places]
            if int(decimals) > 0:
                res = res + decimal_separator + decimals
        res += consts.tens[unit][idx]
    return minus + res


def _join(separator, lst):
    return separator.join([k for k in lst if k != ''])



import jphelper.constant as const


def is_hiragana(text):
    """Check if all characters in text are hiragana."""
    res = True
    for t in text:
        if t not in const.HIRAGANA_SET:
            res = False
            break
    return res


def is_katakana(text):
    """Check if all characters in text are katakana."""
    res = True
    for t in text:
        if t not in const.KATAKANA_SET:
            res = False
            break
    return res


def is_kana(text):
    """Check if all characters in text are hiragana or katakana."""
    res = True
    for t in text:
        if t not in const.HIRAGANA_SET and t not in const.KATAKANA_SET:
            res = False
            break
    return res


def is_kanji(text, start_block=u'\u4e00', end_block=u'\u9fff'):
    """Check if all characters in text are kanji character. [CJK Unified Ideographs 4E00-9FFF Common]"""
    res = True
    for t in text:
        if t < start_block or t > end_block:
            res = False
            break
    return res


def match_reading(expression, reading, space=' ', post_space=''):
    """Match kanji in expression with its corresponding reading.

    :param str expression: Text contains mixture of kanji, hiragana, katakana, etc.
    :param str reading: Text equal to expression but using kana in place of kanji.
    :param str space: Character(s) to be inserted before kanji[furigana] group
    :param str post_space: Character(s) to be inserted after kanji[furigana] group
    :return: Text from expression with furigana attached after kanji.
    :rtype: str
    """
    # assert is_kana(reading)  # reading must be hiragana or katakana
    assert len(expression) > 0 and len(reading) > 0  # non empty string

    # if length of reading text less than expression (not possible), return expression
    if len(reading) < len(expression):
        reading = ''

    res, r_, k_, idx = '', '', '', len(expression) - 1
    _reading = reading[::-1]

    for r in _reading:
        if idx >= 0:
            if r == expression[idx]:
                if k_ == '' and r_ == '':
                    res = r + res
                else:
                    res = r + space + k_ + '[' + r_ + ']' + post_space + res
                    k_, r_ = '', ''
                idx -= 1
            else:
                if is_kanji(expression[idx]):  # if expression[idx] is KANJI_SET
                    k_ = expression[idx] + k_
                    idx -= 1
                    r_ = r + r_
                elif is_kana(expression[idx]):
                    r_ = r + r_
                else:
                    res = r + res
                    idx -= 1
        else:
            if r_ == '' and k_ == '':
                res = r + res
            else:
                r_ = r + r_
    if k_ != '':
        res = k_ + '[' + r_ + ']' + post_space + res
        r_ = ''
    if r_ != '':
        res = r_ + res
    return res if res != '' else expression  # return expression if no match or expression is kana


def kanaize(text, to_katakana=False):
    """Convert romaji to hiragana or katakana equivalent.

    :param str text: Romaji text.
    :param bool to_katakana: Result text will be in katakana if set True.
    :return: Kana version from input text.
    :rtype: str
    """
    k = 1 if to_katakana else 0
    # check long vowel
    ending = 'u' if not to_katakana else const.katakana_long_vowel
    if 'ō' in text:
        text = str.replace(text, 'ō', 'o' + ending)
    if 'ū' in text:
        text = str.replace(text, 'ū', 'u' + ending)
    if '-' in text and to_katakana:
        text = str.replace(text, '-', ending)
    # check double consonant
    for c in const.double_consonants:
        if c in text:
            text = str.replace(text, c, ('xtsu' + c[0]))
    # check small y
    for c in const.small_y:
        if c in text:
            text = str.replace(text, c, const.small_y[c])
    # change
    for c in const.small_kana:
        text = str.replace(text, c, const.small_kana[c][k])
    for c in const.kana2:
        text = str.replace(text, c, const.kana2[c][k])
    for c in const.kana:
        text = str.replace(text, c, const.kana[c][k])
    for c in const.vowel_kana:
        text = str.replace(text, c, const.vowel_kana[c][k])
    # replace n
    text = str.replace(text, 'nn', const.kana_n[k])
    text = str.replace(text, 'n', const.kana_n[k])
    # clean consonant
    check = 'kstnhmyrwgzjdbp'
    for c in check:
        text = str.replace(text, c, 'xtsu')
    # small kana
    for c in const.small_kana:
        text = str.replace(text, c, const.small_kana[c][k])
    return text



from setuptools import setup, find_packages
import unittest
import os.path


def get_test_suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('jphelper.tests', pattern='test_*.py')
    return test_suite


with open('README.rst', encoding='utf-8') as f:
    long_desc = f.read()


setup(
    name='jphelper',
    packages=find_packages(exclude=['*.tests', '*.tests.*', 'tests.*', 'tests']),
    version='0.9.1',
    description='A collection of japanese text and number manipulation tools.',
    author='Michael Devara',
    author_email='michael.devara@gmail.com',
    test_suite='setup.get_test_suite',
    keywords=['japanese', 'utility', 'utils', 'tools'],
    license='MIT',
    platforms=['any'],
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Environment :: Other Environment',
        'Natural Language :: Japanese',
        'Operating System :: OS Independent',
        'Topic :: Utilities'
    ],
    url='https://github.com/michaeldvr/jphelper',
    long_description=long_desc,
    long_description_content_type='text/markdown; charset=UTF-8; variant=GFM',
    python_requires='>=3'
)
